import logging
import re
#account root model
class RootModel():
    def __init__(self,aggreement_value):
        try:
            self.aggreement_value =aggreement_value
            self.aggreement_number =''
            self.aggreement_counter =''

            self.parse_aggreement_value()
        except Exception as e:
            logging.error("Error"+str(e))


    #parse
    def parse_aggreement_value(self):
        try:
            ln_index = -1
            limit = 0
            copy_all =False
            for v_index in range(len(self.aggreement_value) - 1, 0, -1):
                let = self.aggreement_value[v_index]
                if re.match("^[A-Za-z0-9]*$", let):
                    ln_index+=1
                    ln_index=v_index
                else:
                    break

                if (limit>2):
                    ln_index=len(self.aggreement_value)
                    copy_all=True
                    break
                limit+=1

            if (ln_index>1):
                if (copy_all==False):
                    self.aggreement_number = str(self.aggreement_value)[0:ln_index-1]
                    l = len(self.aggreement_value)
                    self.aggreement_counter = str(self.aggreement_value)[ln_index:l]

                else:
                    self.aggreement_number = str(self.aggreement_value)[0:ln_index]



            else:
                self.aggreement_number = self.aggreement_value
            p=0

        except Exception as e:
            logging.error("Error"+str(e))



import xml.etree.ElementTree as ET
import models.export_models.export_document as export_document
import modules.xml_parser.xml_processor as xml_processor
import modules.document_converters.document_converter as document_converter
import logging
import modules.io_modues.io_module as io_module
import modules.table_processing.init_document_tables as init_document_tables
import modules.root_processor.root_processor as root_processor
# parse physical file XML
def parse_file(file_path):
    try:
        tree = ET.parse(file_path)
        root = tree.getroot()

        document = export_document.ExportDocument()
        document.xml_document_tables =init_document_tables.init_document_tables(root)
        document.init_document_elements()
        xml_processor.process_document(root, document)
        result =document_converter.convert_document_to_sheet_format(document)
        logging.info("File parse  successfully completed")
        io_module.export_json_to_file(result)
        return result
    except Exception as e:
        logging.error("Error. " + str(e))


# parse data string
def parse_data_string(data):
    try:
        tree = ET.fromstring(data)
        root = tree.getroot()

        document = export_document.ExportDocument()
        document.xml_document_tables = init_document_tables.init_document_tables(root)
        document.init_document_elements()
        xml_processor.process_document(root, document)
        result = document_converter.convert_document_to_sheet_format(document)
        logging.info("File parse  successfully completed")
        io_module.export_json_to_file(result)
        return result
        pass
    except Exception as e:
        logging.error("Error. " + str(e))



import logging


# get to contracts without counter
def get_to_contracts_without_counter(root_models):
    try:
        root_contracts = []

        for root_model in root_models:
            if (root_model.aggreement_counter == ''):
                root_contracts.append(root_model)

        clean_data = []

        for root_model in root_models:
            found = False
            for root_contract in root_contracts:
                if (root_model.aggreement_value == root_contract.aggreement_value):
                    found = True
                    break

            if (found == False):
                clean_data.append(root_model)

        return root_contracts,clean_data

    except Exception as e:
        logging.error("Error. " + str(e))



#get to child contracts
def get_child_contracts(root_contracts, contracts):
    try:
        results = []

        contract_added = []
        for root_contract in root_contracts:
            child_contracts = []
            for contract in contracts:
                if (str(contract.aggreement_number)==str(root_contract.aggreement_number)):
                    child_contracts.append(contract)
                    contract_added.append(child_contracts)

            results.append([root_contract, child_contracts])

        #detect not comparer contracts

        ext_contracts = []
        for contract in contracts:
            found = False
            for result in results:
                eq_contracts = result[1]
                if (len(eq_contracts)>0):
                    for eq_contract in eq_contracts:
                        if (contract.aggreement_value==eq_contract.aggreement_value):
                            found = True
                            break

            if (found==False):
                ext_contracts.append(contract)





        return results,ext_contracts
    except Exception as e:
        logging.error("Error. " + str(e))

#group extension contracts
def group_ext_contracts(contracts):
    try:
        aggreement_roots = []

        for contract in contracts:
            t = contract.aggreement_number in aggreement_roots
            if (t==False):
                aggreement_roots.append(contract.aggreement_number)


        aggreement_roots.sort()

        groups =[]
        for aggreement_root in aggreement_roots:
            group =[]
            for contract in contracts:
                if (contract.aggreement_number==aggreement_root):
                    group.append(contract)

            if (len(group)>0):
                groups.append(group)
        return groups
    except Exception as e:
        logging.error("Error. " + str(e))

#check if all counters is int
def check_sort_counter_int(counters):
    try:
        result = []
        for counter in counters:
            i_counter = int(counter)
            result.append(i_counter)


        result.sort()
        return result,True

    except Exception as e:
        logging.error("Error. " + str(e))
        return None,False

#sort contracts by counters
def sort_contracts(contracts):
    try:
        contract_counters = []
        for contract in contracts:
            contract_counters.append(contract.aggreement_counter)
        sorted_contract_counters, is_int = check_sort_counter_int(contract_counters)

        if (sorted_contract_counters==None):
            return contracts
        sorted_values = []


        for sorted_contract in sorted_contract_counters:

            for contract in contracts:
                if (str(contract.aggreement_counter)==str(sorted_contract)):
                    sorted_values.append(contract)

        return sorted_values
    except Exception as e:
        logging.error("Error. " + str(e))
        return contracts

# order root models
def order_roots(root_models):
    try:
        roots = []
        # get to head contracts
        root_contracts,clean_data = get_to_contracts_without_counter(root_models)
        child_contracts,ext_contracts = get_child_contracts(root_contracts,clean_data)

        ext_contract_groups = group_ext_contracts(ext_contracts)


        for child_contract in child_contracts:
            contracts =child_contract[1]

            if (len(contracts)>0):
                _contracts = sort_contracts(contracts)
                child_contract[1] = _contracts

        for ext_contract_group in ext_contract_groups:

            ext_contract_group = sort_contracts(ext_contract_group)

            t=0


        for child_contract in child_contracts:
            roots.append([child_contract, True])


        for ext_contract_group in ext_contract_groups:
            roots.append([ext_contract_group, False])

        return roots
    except Exception as e:
        logging.error("Error. " + str(e))



import logging
import modules.xml_parser.xml_table_processor_helpers as xml_helpers
import modules.group_data_modules.group_table_data as group_table_data

#parse xml file
def recursive_navigate(node, paths, index):
    try:
        r =node
        for result in node.findall(paths[index]):
            if (index<len(paths)-1):
                return recursive_navigate(result,paths,index+1)
                # break
            else:
                return result
        return None
    except Exception as e:
        logging.error("Error. " + str(e))

#recursive search node
def recursive_node_navigate(node, paths, index):
        try:
            r = node
            for result in node.findall(paths[index]):
                if (index < len(paths) - 1):
                    return recursive_node_navigate(result, paths, index + 1)
                    # break
                else:
                    return node

            return None
        except Exception as e:
            logging.error("Error. " + str(e))

#extract all values from single node
def extract_all_items_from_node(node, name):
    try:
        return node.findall(name)

    except Exception as e:
        logging.error("Error. " + str(e))


#proces table rows by column and conditions
def process_table_rows(root,table,  xml_rows):
    try:
        index =1
        for xml_row in xml_rows:
            row = table.init_row(index,table.columns)
            for column in table.columns:
                #check extra field or not
                if (column.extra_table_field==True):
                    value = xml_helpers.extract_data_from_extra_table_field(root, column)
                else:
                    value = xml_helpers.extract_data_from_table_row(xml_row,column)
                row.set_cell_value_by_index(column.index,value)
        return
    except Exception as e:
        logging.error("Error. " + str(e))

#process XML table
def process_table(root, document):
    try:
        index =0
        #detect table body
        for table in document.xml_document_tables:
            paths = table.table_path
            last_element = paths[len(paths)-1]
            xml_table_body_node = recursive_node_navigate(root,paths,0)
            xml_table_body_rows = extract_all_items_from_node(xml_table_body_node,last_element)
            process_table_rows(root, table,xml_table_body_rows)
            table = group_table_data.group_data_table(table)


            document.xml_document_tables[index] = table

            index+=1
        return None
    except Exception as e:
        logging.error("Error. " + str(e))


import io, json
import models.settings.config as config
import logging
#export json to file
def export_json_to_file(data):
    try:
        with open(config.export_json_file_full_path, 'w') as f:
            json.dump(data, f, ensure_ascii=False)
        logging.info("Data successfully exported")
    except Exception as e:
        logging.error("Error. " + str(e))



import models.processing_tables.processing_table_models as processing_table_models
import logging


# init existing table
def init_existing_contracts_table(root):
    try:

        table_path = ['Result', 'Root', 'ExistingContracts', 'Contract']
        table = processing_table_models.ProcessingTable("Действующие договора", root, table_path,
                                                        hidden_columns_in_groups=["Name", "SubjectRole",
                                                                                  "FinancialInstitution"])

        # column Наименование Заемщика/гаранта/ созаемщика
        table.init_column("Наименование Заемщика/гаранта/созаемщика", "Name",
                          [
                              [
                                  ['Result', 'Root', 'Header', 'Surname'],
                                  ['Result', 'Root', 'Header', 'Name'],
                                  ['Result', 'Root', 'Header', 'FathersName']
                              ],
                              [
                                  ['Result', 'Root', 'Header', 'NameNative']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(1), True
                          )

        # column Роль Субъекта (гарант, созаемщик, заемщик)
        table.init_column("Роль субъекта", "SubjectRole",
                          [
                              [
                                  ['SubjectRole']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, True),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Наименование Банка
        table.init_column("Наименование Банка", "FinancialInstitution",
                          [
                              [
                                  ['FinancialInstitution'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Тип КЛ
        table.init_column("Тип КЛ", "TypeOfFoundingn",
                          [
                              [
                                  ['TypeOfFounding'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Номер договора
        table.init_column("Номер договора", "AgreementNumber",
                          [
                              [
                                  ['AgreementNumber'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Лимит/ Общая сумма кредита (тыс. тенге)
        table.init_column("Лимит/ Общая сумма кредита (тыс. тенге)", "TotalAmount",
                          [
                              [
                                  # ['CreditLimit'],
                                  ['TotalAmount']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Наименование Заемщика/гаранта/ созаемщика (тыс. тенге)
        table.init_column("Наименование Заемщика/гаранта/ созаемщика (тыс. тенге)", "OutstandingAmount",
                          [
                              [
                                  ['OutstandingAmount']

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False)

        # column Сумма ежемесячного платежа, тыс. тенге
        table.init_column("Сумма ежемесячного платежа, тыс. тенге", "MonthlyInstalmentAmount",
                          [
                              [
                                  ['MonthlyInstalmentAmount'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Дата открытия кредитной линии, ДД.ММ.ГГГГ
        table.init_column("Дата открытия кредитной линии, ДД.ММ.ГГГГ", "DateOfCreditStart",
                          [
                              [
                                  ['DateOfCreditStart'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Дата закрытия кредитной линии, ДД.ММ.ГГГГ
        table.init_column("Дата закрытия кредитной линии, ДД.ММ.ГГГГ", "DateOfCreditEnd",
                          [
                              [
                                  ['DateOfCreditEnd'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Период доступности ДД.ММ.ГГГГ
        table.init_column("Период доступности ДД.ММ.ГГГГ", "AvailableDate",
                          [
                              [
                                  ['AvailableDate'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Ставка вознаграждения (при наличии информации)
        table.init_column("Ставка вознаграждения (при наличии информации) ", "NominalRate",
                          [
                              [
                                  ['NominalRate'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Цель
        table.init_column("Цель", "CreditObject/PurposeOfCredit",
                          [
                              [
                                  ['CreditObject'],
                                  ['PurposeOfCredit']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, True),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Продолжительность просрочки, дней
        table.init_column("Продолжительность просрочки, дней", "OverdueAmountMaxCount",
                          [
                              [
                                  ['OverdueAmountMaxCount'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Максимальная длина одной просрочки (max)
        table.init_column("Максимальная длина одной просрочки (max)", "NumberOfOverdueInstalmentsMax",
                          [
                              [
                                  ['NumberOfOverdueInstalmentsMax'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Наличие текущей просрочки, в днях
        table.init_column("Наличие текущей просрочки, в днях", "NumberOfOverdueInstalments",
                          [
                              [
                                  ['NumberOfOverdueInstalments'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Просроченная сумма, тыс. Тенге (макс.)
        table.init_column("Просроченная сумма, тыс. Тенге (макс.)", "NumberOfOverdueInstalmentsMaxAmount",
                          [
                              [
                                  ['NumberOfOverdueInstalmentsMaxAmount'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Просроченная сумма, тыс. Тенге (макс.)
        table.init_column("Количество месяцев, в которых была зафиксирована просрочка", "MonthCountInstalments",
                          [
                              [
                                  ['-'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Количество просрочек более 30 дней
        table.init_column("Количество просрочек более 30 дней", "CountInstalmentsIn30Days",
                          [
                              [
                                  ['-'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Гаранты/созаемщики
        table.init_column("Гаранты/созаемщики", "Garants",
                          [
                              [
                                  ['-'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Вид обеспечения
        table.init_column("Вид обеспечения", "TypeOfGuarantee",
                          [
                              [
                                  ['Collateral', 'TypeOfGuarantee'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Стоимость обеспечения тыс. тенге
        table.init_column("Стоимость обеспечения тыс. тенге", "ValueOfGuarantee",
                          [
                              [
                                  ['Collateral', 'ValueOfGuarantee'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        return table
    except Exception as e:
        pass


# init exsisting table
def init_terminated_contracts_table(root):
    try:

        table_path = ['Result', 'Root', 'TerminatedContracts', 'Contract']
        table = processing_table_models.ProcessingTable("Завершенные договора", root, table_path,
                                                        hidden_columns_in_groups=["Name", "SubjectRole",
                                                                                  "FinancialInstitution"])

        # column Наименование Заемщика/гаранта/ созаемщика
        table.init_column("Наименование Заемщика/гаранта/созаемщика", "Name",
                          [
                              [
                                  ['Result', 'Root', 'Header', 'Surname'],
                                  ['Result', 'Root', 'Header', 'Name'],
                                  ['Result', 'Root', 'Header', 'FathersName']
                              ],
                              [
                                  ['Result', 'Root', 'Header', 'NameNative']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(1), True
                          )

        # column Роль Субъекта (гарант, созаемщик, заемщик)
        table.init_column("Роль субъекта", "SubjectRole",
                          [
                              [
                                  ['SubjectRole']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, True),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Наименование Банка
        table.init_column("Наименование Банка", "FinancialInstitution",
                          [
                              [
                                  ['FinancialInstitution'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Тип КЛ
        table.init_column("Тип КЛ", "TypeOfFoundingn",
                          [
                              [
                                  ['TypeOfFounding'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Номер договора
        table.init_column("Номер договора", "AgreementNumber",
                          [
                              [
                                  ['AgreementNumber'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Лимит/ Общая сумма кредита (тыс. тенге)
        table.init_column("Лимит/ Общая сумма кредита (тыс. тенге)", "TotalAmount",
                          [
                              [
                                  # ['CreditLimit'],
                                  ['TotalAmount']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )


        # column Дата открытия кредитной линии, ДД.ММ.ГГГГ
        table.init_column("Дата открытия кредитной линии, ДД.ММ.ГГГГ", "DateOfCreditStart",
                          [
                              [
                                  ['DateOfCreditStart'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Дата закрытия кредитной линии, ДД.ММ.ГГГГ
        table.init_column("Дата закрытия кредитной линии, ДД.ММ.ГГГГ", "DateOfCreditEnd",
                          [
                              [
                                  ['DateOfCreditEnd'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Период доступности ДД.ММ.ГГГГ
        table.init_column("Период доступности ДД.ММ.ГГГГ", "AvailableDate",
                          [
                              [
                                  ['AvailableDate'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # date format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         2),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Ставка вознаграждения (при наличии информации)
        table.init_column("Ставка вознаграждения (при наличии информации) ", "NominalRate",
                          [
                              [
                                  ['NominalRate'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(0), False
                          )

        # column Цель
        table.init_column("Цель", "CreditObject/PurposeOfCredit",
                          [
                              [
                                  ['CreditObject'],
                                  ['PurposeOfCredit']
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, True),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Продолжительность просрочки, дней
        table.init_column("Продолжительность просрочки, дней", "OverdueAmountMaxCount",
                          [
                              [
                                  ['OverdueAmountMaxCount'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Максимальная длина одной просрочки (max)
        table.init_column("Максимальная длина одной просрочки (max)", "NumberOfOverdueInstalmentsMax",
                          [
                              [
                                  ['NumberOfOverdueInstalmentsMax'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),

                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )


        # column Просроченная сумма, тыс. Тенге (макс.)
        table.init_column("Просроченная сумма, тыс. Тенге (макс.)", "NumberOfOverdueInstalmentsMaxAmount",
                          [
                              [
                                  ['NumberOfOverdueInstalmentsMaxAmount'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Просроченная сумма, тыс. Тенге (макс.)
        table.init_column("Количество месяцев, в которых была зафиксирована просрочка", "MonthCountInstalments",
                          [
                              [
                                  ['-'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Количество просрочек более 30 дней
        table.init_column("Количество просрочек более 30 дней", "CountInstalmentsIn30Days",
                          [
                              [
                                  ['-'],

                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Гаранты/созаемщики
        table.init_column("Гаранты/созаемщики", "Garants",
                          [
                              [
                                  ['-'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Вид обеспечения
        table.init_column("Вид обеспечения", "TypeOfGuarantee",
                          [
                              [
                                  ['Collateral', 'TypeOfGuarantee'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(0),
                                                                 None, None, None, False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        # column Стоимость обеспечения тыс. тенге
        table.init_column("Стоимость обеспечения тыс. тенге", "ValueOfGuarantee",
                          [
                              [
                                  ['Collateral', 'ValueOfGuarantee'],
                              ]

                          ], 'value',
                          processing_table_models.ConditionValue(None, None, None, 'value', None,
                                                                 processing_table_models.ConvertTypesCollection(1),
                                                                 None, None,
                                                                 [
                                                                     # only numbers
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         1),
                                                                     # money format
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         3),
                                                                     # thousand
                                                                     processing_table_models.OutputValueTypesCollection(
                                                                         4)
                                                                 ],
                                                                 False),
                          processing_table_models.ExtractValueTypesCollection(2), False
                          )

        return table
    except Exception as e:
        pass

# init document processing dtables
def init_document_tables(root):
    try:
        tables = []
        existing_contracts_table = init_existing_contracts_table(root)
        terminated_contracts_table = init_terminated_contracts_table(root)
        tables.append(existing_contracts_table)
        tables.append(terminated_contracts_table)
        return tables
    except Exception as e:
        logging.error("Error initialization. " + str(e))



import os, sys
import modules.logging.logging as logging_config

import json
import logging

# root dir
root_dir = os.path.dirname(os.path.realpath(sys.argv[0]))

# logs dir full path
logs_dir_full_path = ""

# logs dir name
logs_dir_name = "logs"

# configs dir full path
configs_dir_full_path = ""

# configs dir name
configs_dir_name = "configs"

# data dir full path
data_dir_full_path = ""

# data dir name
data_dir_name = "data"

# engine version code
version_code = "0.4"

# build version code
build_version_code = -1

# engine name
engine_name = "PKB Report Builder"

# system name
system_name = "PKB Report Builder Engine"

# show all info when running
show_system_info_on_start = False

# logging config file name
logging_config_file_name = "logging.yaml"

# logging config file full pat
logging_config_file_full_path = ""

# buildcode file name
build_code_file_name = "build_code.json"

# logging config file full pat
build_code_file_full_path = ""

# test parse file
test_parse_file_name = "77873.xml"
# test_parse_file_name = "78477.xml"

# test parse file full path
test_parse_file_full_path = ""

# configs dir full path
exports_dir_full_path = ""

# configs dir name
exports_dir_name = "exports"

# export json file full path
export_json_file_full_path = ""

# export json file name
export_json_file_name = "data.json"

# all versions
versions_container = [
    ["0.1", "Leonardo"],
    ["0.2", "Donatello"],
    ["0.3", "Raphael"],
    ["0.4", "Michelangelo"],
    ["0.5", "Splinter"],

]


# --------------- methods---------------#
# load and increase build version
def load_build_version():
    try:
        with open(build_code_file_full_path) as f:
            data = json.load(f)
            global build_version_code
            build_version_code = int(data["build_version_code"])
            build_version_code += 1
            data["build_version_code"] = build_version_code

        with open(build_code_file_full_path, "w") as jsonFile:
            json.dump(data, jsonFile)
    except Exception as e:
        logging.error("Error. " + str(e))

#get to version name
def get_version_name():
    try:
        versions = versions_container
        current_version_code = version_code

        for version in versions:
            if (version[0] == current_version_code):
                return version[1]
    except Exception as e:
        logging.error("Error. " + str(e))
        return str(e)

# join system paths
def join_path(paths):
    try:
        path = os.path.join(*paths)
        path = os.path.normpath(path)
        # path =os.path.normcase(path)
        return path
    except Exception as e:
        return str(e)


# show start info
def show_system_info():
    try:
        print("Start init configuration")
        print("Root dir : " + root_dir)
        print("System name : " + system_name)
        print("Engine name : " + engine_name)
        print("Engine version code: " + version_code)
        print("Engine version name: " + get_version_name())
        print("Build version code: " + str(build_version_code))

        pass
    except Exception as e:
        pass

# check single dir
def check_single_dir(dir_path):
        try:
            dir_exists = os.path.exists(dir_path)

            if (dir_exists == False):
                os.mkdir(dir_path)
                logging.info("Create directory " + dir_path)
        except Exception as e:
            logging.error(str(e))

# check system dirs
def check_dirs():
        try:
            check_single_dir(configs_dir_full_path)
            check_single_dir(logs_dir_full_path)
            check_single_dir(data_dir_full_path)
            check_single_dir(exports_dir_full_path)
        except Exception as e:
            logging.error(str(e))

# config system paths
def config_paths():
    try:
        global configs_dir_full_path
        configs_dir_full_path = join_path([root_dir, configs_dir_name])

        global logging_config_file_full_path
        logging_config_file_full_path = join_path([configs_dir_full_path, logging_config_file_name])

        global build_code_file_full_path
        build_code_file_full_path = join_path([configs_dir_full_path, build_code_file_name])

        global logs_dir_full_path
        logs_dir_full_path = join_path([root_dir, logs_dir_name])

        global data_dir_full_path
        data_dir_full_path = join_path([root_dir, data_dir_name])

        global test_parse_file_full_path
        test_parse_file_full_path = join_path([data_dir_full_path, test_parse_file_name])

        global exports_dir_full_path
        exports_dir_full_path = join_path([root_dir, exports_dir_name])

        global export_json_file_full_path
        export_json_file_full_path = join_path([exports_dir_full_path, export_json_file_name])

        t = 0
        pass
    except Exception as e:
        pass


# init config
def init_config():
    try:
        config_paths()
        check_dirs()
        load_build_version()
        logging_config.setup_logging(default_path=logging_config_file_full_path)

        if (show_system_info_on_start == True):
            show_system_info()

        logging.info("Config init successful")

    except Exception as e:
        logging.error(str(e))
        pass



import logging

#xml navigator additional params
class XmlNavigatorAdditionalParams():
    def __init__(self):
        try:

            complex_value_attribute =False
            complex_value_attribute_paths =[]



            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))



#xml tree navigator
class XmlNavigator():
    #constructor
    def __init__(self,paths,value_attribute, convert_types, style,additional_params=None):
        try:
            self.paths = paths
            self.value_attribute = value_attribute
            self.convert_types = convert_types
            self.style =style

            self.additional_params =additional_params
        except Exception as e:
            logging.error("Error initialization. " + str(e))



import xml.etree.ElementTree as ET

#find block
def find_block(current_block, element_name):
    try:
        results =[]
        for result in current_block.findall(element_name):
            results.append(result)
        return results
    except Exception as e:
        pass


import logging
import modules.xml_parser.xml_table_processor as xml_table_processor
import models.tree_models.xml_navigator as navigator
import models.export_models.export_styles as export_styles
#parse xml file
def recursive_navigate(node, paths, index):
    try:
        r =node
        for result in node.findall(paths[index]):
            if (index<len(paths)-1):
                return recursive_navigate(result,paths,index+1)
                # break
            else:
                return result

        return None
    except Exception as e:
        logging.error("Error. " + str(e))

#process cell
def process_cell(root, cell):
    try:
        cell_value = ""
        index =0
        for path in cell.paths:
            node = recursive_navigate(root,path,0)
            if (node==None):
                return None
            value = node.get(cell.value_attribute)
            cell_value+=value
            if (index<len(cell.paths)-1):
                cell_value+='\n'
            index+=1

        cell.set_cell_value(cell_value)
        return cell

    except Exception as e:
        logging.error("Error. " + str(e))
        return None


#process single row
def process_row(root,export_element):
    try:
        for cell in export_element.row.cells:
            res =process_cell(root,cell)
            if (res==None):
                export_element.include_in_report =False
        pass
    except Exception as e:
        logging.error("Error. " + str(e))


def check_field(element,name):
    try:
        v = getattr(element,name)

        return v
        pass
    except Exception as e:
        return None


#add element to document
def add_element_to_document(document,style,values):
    try:

        t=0
        navigate_params = navigator.XmlNavigator('','','',style)
        document.add_export_element([0,'',navigate_params])

        last_element = document.export_elements[len(document.export_elements)-1]


        last_element.row.set_row_style(style.bgc,style.color,style.ta,style.fz,style.fm,style.fw,style.fs,style.u,style.ff)

        str_values =[]

        for value in values:

            is_str=type(value) is str

            element_value=''
            if (is_str==False):
                element_value = check_field(value,'title')

                if (element_value==None):
                    element_value = check_field(value,'value')

            else:
                element_value = str(value)

            str_values.append(element_value)

        last_element.row.init_value_cells(navigate_params, str_values)
        last_element.row.set_row_cells_style()

        t=0

        pass
    except Exception as e:
        pass

#check if row is empty
def check_clean_row(row):
    try:
        for cell in row.cells:
            if (cell.value!=''):
                return False

        return True
        pass
    except Exception as e:
        logging.error("Error. " + str(e))
        return True

#clean empty rows
def clean_empty_rows(document):
    try:
        clean_export_elements =[]

        for export_element in document.export_elements:
            if(check_clean_row(export_element.row)==False):
                clean_export_elements.append(export_element)

        document.export_elements = []
        document.export_elements = clean_export_elements

        t=0
        pass

    except Exception as e:
        logging.error("Error. " + str(e))


#generate group rows
def generate_group_rows(document,table):
    try:
        elements_count = len(document.export_elements)

        for group_row in table.group_rows:
            n_group = [group_row[0]+elements_count+1,group_row[1]+elements_count+1]
            document.group_rows.append(n_group)

    except Exception as e:
        logging.error("Error. " + str(e))

#process table
def process_table(document):
    try:

        clean_empty_rows(document)
        add_element_to_document(document, export_styles.row_title_style, [''])

        for table in document.xml_document_tables:
            add_element_to_document(document, export_styles.row_title_style,[table.title])

            generate_group_rows(document,table)
            add_element_to_document(document,export_styles.table_header_style,table.columns)

            for row in table.rows:

                if (row.not_classificated==True):
                    add_element_to_document(document, export_styles.table_not_classificated_rows, row.cells)
                    continue

                if (row.is_root==False):
                    add_element_to_document(document, export_styles.table_rows_content_style, row.cells)
                else:
                    add_element_to_document(document, export_styles.table_rows_root_content_style, row.cells)
            pass

            add_element_to_document(document, export_styles.row_title_style,[''])



        pass
    except Exception as e:
        logging.error("Error. " + str(e))

#process document
def process_document(root, document):
    try:
        for export_element in document.export_elements:
            type_id = export_element.type.value

            if (type_id==0):
                process_row(root,export_element)
            elif (type_id==1):
                pass

        xml_table_processor.process_table(root, document)
        process_table(document)
        pass
    except Exception as e:
        logging.error("Error. " + str(e))


import logging
import models.export_models.export_document_elements as export_document_models
import models.tree_models.xml_navigator as xml_navigator
import models.export_models.export_styles as styles
# export  document
class ExportDocument():
    # constructor
    def __init__(self):
        try:
            # export elements
            self.export_elements = []
            self.xml_document_tables = []
            self.group_rows = []
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # add export elements
    def add_export_element(self, params):
        try:
            type = params[0]
            name = params[1]
            navigate_params = params[2]
            element = export_document_models.ExportDocumentElement(type, name)

            if (type == 0):
                element.init_element_row(navigate_params)
            elif (type == 1):
                element.init_element_table(navigate_params)

            self.export_elements.append(element)
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    #init single field elements
    def init_document_field_elements(self):
        try:
            params = [
                # преступники
                [
                    0, 'MvdCriminalInvestigations', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'MvdCriminalInvestigations']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        ['Result', 'Root', 'PublicSources', 'MvdCriminalInvestigations', 'Status'],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # пропавшие без вести
                [
                    0, 'MvdMissingInvestigations', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'MvdMissingInvestigations']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'MvdMissingInvestigations', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # террористы
                [
                    0, 'TerrorList', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'TerrorList']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'TerrorList', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # задолженность > 6 месяцев
                [
                    0, 'Areears', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'Areears']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'Areears', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # лжепредпринимательская деятельность
                [
                    0, 'FalseBusi', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'FalseBusi']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'FalseBusi', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # банкроты
                [
                    0, 'Bankruptcy', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'Bankruptcy']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'Bankruptcy', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # гос закупки
                [
                    0, 'RNUGosZakup', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'RNUGosZakup']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'RNUGosZakup', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # преступники по правовой статистике
                [
                    0, 'QamqorList', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'QamqorList']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'QamqorList', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # преступники по алиментам
                [
                    0, 'QamqorAlimony', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'QamqorAlimony']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'QamqorAlimony', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ],
                # преступники по линии МФ РК
                [
                    0, 'KgdWanted', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'KgdWanted']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'KgdWanted', 'Status']
                        ]
                        ,
                        'value',
                        [0],
                        styles.row_value_style
                    )
                ]
                ]
            ]

            return params
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    def init_document_table_elements(self):
        try:
            params =[
                # действующие договора
                [
                    1, 'ExistingContracts', [
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'ExistingContracts', 'Contract']
                        ],
                        'title',
                        [0],
                        styles.row_title_style
                    ),
                    xml_navigator.XmlNavigator(
                        [
                            ['Result', 'Root', 'PublicSources', 'KgdWanted', 'Status']
                        ],
                        'value',
                        [0],
                        styles.row_title_style
                    )
                ]
                ],
            ]
            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))


    # init document schemas elements
    def init_document_elements(self):
        try:
            #generate field params
            field_element_params =self.init_document_field_elements()
            for param in field_element_params:
                self.add_export_element(param)

        except Exception as e:
            logging.error("Error initialization. " + str(e))



import logging
import re
#check not classificated rows
def group_not_singled_roots(roots, single_roots):
    try:
        not_classificated_table = []

        pseudo_roots =[]

        for single_root in  single_roots:

            value = single_root
            ln_index =-1
            limit =0
            for v_index in range(len(value)-1,0,-1):

                let = value[v_index]

                if re.match("^[A-Za-z0-9_-]*$", let):
                    ln_index+=1
                    ln_index=v_index
                else:
                    break

                if (limit>2):
                    ln_index=len(value)
                    break
                limit+=1

            p = str(value)[0:ln_index]
            if (len(p)>1):
                pseudo_roots.append(p)


        clean_pseudo_roots = []

        for pseudo_root in  pseudo_roots:
            t = pseudo_root in clean_pseudo_roots
            if (t==False):
                clean_pseudo_roots.append(pseudo_root)

        result_pseudo_roots =[]
        result_single_roots = []

        for clean_pseudo_root in clean_pseudo_roots:
            t = clean_pseudo_root in single_roots
            if (t==True):
                result_single_roots.append(clean_pseudo_root)
            else:
                result_pseudo_roots.append(clean_pseudo_root)




        return result_single_roots,result_pseudo_roots
        t=0
        pass
    except Exception as e:
        logging.error("Error"+str(e))


import logging
import models.export_models.export_document_report_model as export_document_report_model
import json

# set zero row settings
def set_zero_row(document, sheet_id, widths):
    try:
        id = 1
        for width in widths:
            json = export_document_report_model.ExportDocumentReportCellContent("", width=width, height=45)
            cell = export_document_report_model.ExportDocumentReportCell(sheet_id, 0, id, json)
            document.cells.append(cell)
            id += 1
    except Exception as e:
        logging.error("Error. " + str(e))

#set zero column settings
def set_zero_column(document):
    try:
        json = export_document_report_model.ExportDocumentReportCellContent("", height=45)
        cell = export_document_report_model.ExportDocumentReportCell(1, 0, 0, json)
        document.cells.append(cell)
    except Exception as e:
        logging.error("Error. " + str(e))

# get max width column
def get_max_width(export_document):
    try:
        columns = []
        rows = []
        for element in export_document.export_elements:
            if (element.type.value == 0):
                rows.append(element.row)
            else:
                rows.append(element.table.header_row)
                for r in element.table.rows:
                    rows.append(r)
        for r in rows:
            id = 0
            for c in r.cells:
                l = len(str(c.value))
                if (l > 0):
                    if (len(columns) < id + 1):
                        columns.append(l)
                    else:
                        if (columns[id] < l):
                            columns[id] = l

                id += 1
        widths = []
        for c in columns:
            w = c * 6
            if (w > 240):
                w = 240
            widths.append(w)
        return widths
        pass
    except Exception as e:
        logging.error("Error. " + str(e))


# init single row
def init_row(document, row, row_index, show_grid):
    try:
        id = 1
        for cell in row.cells:
            # content_type_id = cell.content_type.value
            style = cell.style

            back_color = style.bgc
            font_color = style.color
            font_family = style.ff
            font_style = style.fs
            font_size = style.fz
            text_align = style.ta

            font_weight = style.fw
            if (font_style == "bold"):
                font_weight = "bold"
                font_style = "normal"

            if (font_style == "bolditalic"):
                font_weight = "bold"
                font_style = "italic"

            if (font_style == "italic"):
                font_weight = "normal"
                font_style = "italic"

            fm = ""
            # if (
            #                         content_type_id == 4 or content_type_id == 6 or content_type_id == 7 or content_type_id == 9 or content_type_id == 11 or content_type_id == 12):
            #     fm = "money||2|none"

            cell_content = export_document_report_model.ExportDocumentReportCellContent(
                data=cell.value,
                bgc=back_color,
                color=font_color,
                ta=text_align,
                fz=font_size,
                ff=font_family,
                fw=font_weight,
                fs=font_style,
                show_grid=show_grid,
                fm=fm,
                contains_data=True

            )
            report_cell = export_document_report_model.ExportDocumentReportCell(1, row_index, id, cell_content)
            document.cells.append(report_cell)
            id += 1
    except Exception as e:
        logging.error("Error. " + str(e))


# init rows
def init_rows(export_document, document):
    try:
        rows = []
        for element in export_document.export_elements:
            if (element.type.value == 0):
                rows.append(element.row)
            else:
                rows.append(element.table.header_row)
                for r in element.table.rows:
                    rows.append(r)

        row_index = 1
        for row in rows:
            show_grid = True
            init_row(document, row, row_index, show_grid)
            row_index += 1
    except Exception as e:
        logging.error("Error. " + str(e))


# init document cells
def init_document_cells(export_document, document):
    try:
        width_columns = get_max_width(export_document)
        set_zero_row(document, 1, width_columns)
        set_zero_column(document)
        init_rows(export_document, document)
    except Exception as e:
        logging.error("Error. " + str(e))


# convert document to sheet format
def convert_document_to_sheet_format(export_document):
    try:
        document_report_model = export_document_report_model.ExportDocumentReportModel("Отчет ПКБ")
        id = 1
        sheets = ["Детализация отчета ПКБ"]
        for sheet in sheets:
            document_report_model.add_sheet(id, sheet)
            id += 1

        document_groups = []

        for group_row in export_document.group_rows:
            document_groups.append(export_document_report_model.ExportDocumentGroup(1,group_row))


        # json_string = json.dumps(document_groups)

        json_string = json.dumps([ob.__dict__ for ob in document_groups])

        json_string = json_string.replace("\"", '')


        fl = export_document_report_model.ExportDocumentReportFloatings(1, "rowGroups", "rowgroup", json_string)
        document_report_model.floatings.append(fl)

        init_document_cells(export_document, document_report_model)
        return document_report_model.toJSON()
    except Exception as e:
        logging.error("Error. " + str(e))



import click

@click.group()
def data_parse_commands_cli():
    pass

#get data from file
@data_parse_commands_cli.command()
@click.option('--file_name', required=True, type=str)
#@click.option('--age', required=False, type=int)
def get_data_from_file(file_name):
    click.echo("Name = " + file_name)


# import sys
# import modules.shell_starter_modules.shell_starter_module as shell_starter
# from models import export_models
# from .models
from .models import settings
from .modules import logging
# from .models.export_models import export_styles as styles
# import modules.xml_parser.xml_parser as xml_parser
import datetime
import logging

# init system settings
def init_system():
    try:

        # init config
        settings.init_config()
        #init styles
        # styles.init_styles()
    except Exception as e:
        logging.error("Crash init system procedure. " + str(e))

# #init lib
# def init_lib():
#     try:
#
#         pass
#     except Exception as e:
#         logging.error("Crash init system procedure. " + str(e))
#
# #parse xml file
# def parse_xml_file(file_path):
#     try:
#         init_system()
#         return xml_parser.parse_file(file_path)
#         pass
#     except Exception as e:
#         logging.error("Crash init system procedure. " + str(e))
#
#
# #parse xml string
# def parse_json_string_data(xml_data):
#     try:
#         init_system()
#         return xml_parser.parse_data_string(xml_data)
#         pass
#     except Exception as e:
#         logging.error("Crash init system procedure. " + str(e))


# entry point project
#
# if __name__ == '__main__':
#     try:
#         result =""
#         t = datetime.datetime.now()
#         start_time = datetime.datetime.now()
#         init_system()
#         args_count = len(sys.argv)
#         if (args_count > 1):
#             shell_starter.cli()
#         else:
#             xml_parser.parse_file(config.test_parse_file_full_path)
#
#         execution_time = datetime.datetime.now() - start_time
#         logging.info("Execution time = " + str(execution_time))
#
#     except Exception as e:
#         logging.error("Error core procedure. " + str(e))



import logging
from enum import Enum


# collection of value data types
class ValueDataTypesCollection(Enum):
    SIMPLE_VALUE = 0
    GROUP_VALUE = 1
    ARRAY_VALUE = 2
    CONDITION_VALUE = 3


# convert types collection
class ConvertTypesCollection(Enum):
    WITHOUT = 0
    ONLY_NUMBERS = 1
    DATE = 1


# CONDITION TYPES
class ConditionTypesCollection():
    EQUAL = 0
    NOT_EQUAL = 1
    MORE_THAN = 2
    LESS_THAN = 3
    EXISTS = 4


# OUTPUT|EXTRACTED FORMAT
class OutputValueTypesCollection(Enum):
    WITHOUT = 0
    ONLY_NUMBERS = 1
    DATE = 2
    MONEY_FORMAT = 3
    THOUSAND_FORMAT = 4


# extract value types
class ExtractValueTypesCollection(Enum):
    SINGLE = 0
    JOIN_VALUE = 1
    JOIN_LIST = 2


# condition value
class ConditionValue():
    # constructor
    def __init__(self,
                 # path to xml condition block
                 path_to_condition,
                 # condition value attribute path value in xml block
                 value_for_check_condition_attribute,
                 # path to extracted value
                 path_to_value,
                 # attribute for extracted path value in xml block
                 value_attribute,
                 # ConvertTypesCollection for condition value
                 convert_value_for_checking_type,
                 # ConvertTypesCollection for extract value
                 convert_value_type,
                 # ConditionTypesCollection
                 condition_logic,
                 # condition equal values array
                 condition_equal_values,
                 #output value formats
                 output_value_formats,
                 # is mulitple value or single
                 is_multiple_values=False):
        try:

            self.path_to_condition = path_to_condition
            self.value_for_check_condition_attribute = value_for_check_condition_attribute
            self.path_to_value = path_to_value
            self.value_attribute = value_attribute,
            self.convert_value_for_checking_type = convert_value_for_checking_type
            self.convert_value_type = convert_value_type
            self.condition_logic = condition_logic
            self.condition_equal_values = condition_equal_values
            self.output_value_formats =output_value_formats
            self.is_multiple_values = is_multiple_values

        except Exception as e:
            logging.error("Error initialization. " + str(e))


# processing table cells
class ProcessingTableCell():
    # constructor
    def __init__(self, column):
        try:
            self.value = ""
            self.column = column
            self.index = column.index
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # convert value
    def convert_value(self, value):
        try:
            return value
        except Exception as e:
            logging.error("Error convert cell value. " + str(e))

    # set value
    def set_value(self, value):
        try:
            if (self.column.output_value_type.value != 1):
                self.value = self.convert_value(value)
            else:
                self.value = value
        except Exception as e:
            logging.error("Error set value to cell. " + str(e))


# processing table row
class ProcessingTableRow():
    # constructor
    def __init__(self, index, columns):
        try:
            self.index = index
            self.columns = columns
            self.cells = []
            self.is_root = False
            self.group_id =-1
            self.not_classificated =False
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # self init row cells by column params
    def init_cells_by_columns(self):
        try:
            for column in self.columns:
                self.cells.append(ProcessingTableCell(column))
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    #set cell value by index
    def set_cell_value_by_index(self,index,value):
        try:
            for cell in self.cells:
                if (cell.column.index==index):
                    cell.value = value
                    return
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    #set cells styles
    def set_cells_styles(self,style):
        try:

            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))

# processing table header
class ProcessingTableColumn():
    # constructor
    def __init__(self, index, title, name, value_paths, value_attribute_name, condition, extract_value_type,
                 extra_table_field=False):
        try:
            self.title = title
            self.name = name
            self.index = index
            self.value_paths = value_paths
            self.value_attribute_name = value_attribute_name
            self.extract_value_type = extract_value_type
            self.condition = condition
            self.extra_table_field = extra_table_field
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# class processing table, for complex analyse
class ProcessingTable():
    # constructor
    def __init__(self, title, root_path, table_path, show_title=True,hidden_columns_in_groups=[]):
        try:
            self.title = title
            self.show_title = show_title
            self.table_path = table_path
            self.root_path = root_path
            self.columns = []
            self.rows = []
            self.group_rows = []
            self.hidden_columns_in_groups = hidden_columns_in_groups
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # init single column
    def init_column(self, title, name, value_paths, value_attribute_name, condition, extract_value_type,
                    extra_table_field=False):
        try:
            column_index = len(self.columns) + 1
            column = ProcessingTableColumn(column_index, title, name, value_paths, value_attribute_name, condition,
                                           extract_value_type, extra_table_field)
            self.columns.append(column)

        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # def group rows by column values
    def group_rows_by_values(self, column):
        try:
            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # init table row
    def init_row(self, index, columns):
        try:
            row = ProcessingTableRow(index, columns)
            row.init_cells_by_columns()
            self.rows.append(row)

            return self.rows[len(self.rows) - 1]
        except Exception as e:
            logging.error("Error initialization. " + str(e))



import logging
import copy
import modules.table_processing.not_classificated_records as not_classificated_records
import models.root_models.root_model as root_model
import modules.root_processor.root_processor as root_processor


# check if value is root element in
def check_roots(values, value):
    try:
        if (value == ''):
            return False
        for _value in values:
            if (str(_value) != str(value) and str(_value).startswith(value)):
                return True

        return False
    except Exception as e:
        logging.error("Error. " + str(e))
        return False

        # check if value is root element in


def check_result_roots(values, value):
    try:

        for _value in values:
            if (_value != value and str(value).startswith(_value)):
                return False

        return True


    except Exception as e:
        logging.error("Error. " + str(e))
        return False


# get t contract roots
def get_to_roots(values):
    try:
        root_values = []
        for value in values:
            if (check_roots(values, value) == True):
                root_values.append(value)
        root_values.sort()
        # return root_values
        result_root_values = []
        for root_value in root_values:
            if (check_result_roots(root_values, root_value) == True):
                result_root_values.append(root_value)

        # #clean repea
        return result_root_values
    except Exception as e:
        logging.error("Error. " + str(e))
        return values


# get to single contracts without roots
def get_to_single_contracts(b_values, roots):
    try:
        for root in roots:
            for i in range(0, len(b_values)):
                if (str(b_values[i][0]) == str(root) or str(b_values[i][0]).startswith(str(root))):
                    b_values[i][1] = True

        single_roots = []
        for b_value in b_values:
            if (len(b_value) > 1):
                if (b_value[1] == False):
                    if (b_value[0] != ''):
                        single_roots.append(b_value[0])
        return single_roots
    except Exception as e:
        logging.error("Error. " + str(e))
        return []


# sum chid element
def sum_child_element(rows, group_id, column_id):
    try:
        result = 0
        founded = False
        for row in rows:
            if (row.is_root == False and row.group_id == group_id):
                value = float(row.cells[column_id].value)
                result += value
                founded = True

        if (founded == False):
            return None

        return round(result, 2)
    except Exception as e:
        logging.error("Error. " + str(e))
        return 0


# calculate child elements
def calculate_root_elements(table):
    try:
        column_names = ['TotalAmount', 'OutstandingAmount', 'MonthlyInstalmentAmount']
        for column_name in column_names:

            for row in table.rows:
                if (row.is_root == True):
                    column_index = -1
                    group_id = row.group_id
                    index = 0

                    for cell in row.cells:
                        if (cell.column.name == column_name):
                            column_index = index
                            break
                        index += 1
                    if (column_index == -1):
                        continue
                    sum_rows = sum_child_element(table.rows, group_id, column_index)
                    if (sum_rows != None):
                        row.cells[column_index].value = str(sum_rows)

        t = 0
    except Exception as e:
        logging.error("Error. " + str(e))


# get to table row by agreement number
def get_table_row_by_aggreement_number(table, cell_index, value):
    try:
        row_index = 0
        for row in table.rows:
            if (str(row.cells[cell_index].value) == str(value)):
                return row_index, row
        return -1, None
    except Exception as e:
        logging.error("Error. " + str(e))
        return -1, None


# group data by aggreement number
def group_data_table(table):
    try:
        # clean table
        # table, not_classificate_table = not_classificated_records.check_not_classificated_rows(table)


        column_name = 'AgreementNumber'
        values = []
        first_row = table.rows[0]
        cell_index = 0
        for cell in first_row.cells:
            if (cell.column.name == column_name):
                break
            cell_index += 1

        not_classificated_rows = []
        index = 0
        for row in table.rows:
            if (row.cells[cell_index].value != ''):
                values.append(row.cells[cell_index].value)
            else:
                not_classificated_rows.append(row)

        values.sort()

        sort_rows = []
        for value in values:
            for row in table.rows:
                if (str(row.cells[cell_index].value) == str(value)):
                    sort_rows.append(row)
                    continue

        root_models = []
        for value in values:
            root_models.append(root_model.RootModel(value))

        order_roots = root_processor.order_roots(root_models)

        table.rows = sort_rows

        sort_table = copy.deepcopy(table)
        sort_table.rows = []
        current_group_id = -1
        current_row_index = 1

        for order_root in order_roots:
            value = order_root[0]
            grouping = order_root[1]
            t = 0

            if (grouping == True):
                root_element = value[0]
                child_elements = value[1]

                current_group_id+=1
                # add root
                row_index, root_row = get_table_row_by_aggreement_number(table, cell_index,
                                                                         root_element.aggreement_value)
                start_row_group_index = current_row_index
                end_row_group_index = current_row_index
                root_row.is_root = True
                root_row.group_id = current_group_id
                sort_table.rows.append(root_row)
                current_row_index += 1

                # check childs if need add them
                for child_element in child_elements:
                    row_index, child_row = get_table_row_by_aggreement_number(table, cell_index,
                                                                              child_element.aggreement_value)

                    if (child_row != None):
                        child_row.group_id = current_group_id

                        # hidden_columns_in_groups
                        for cell in child_row.cells:
                            for hidden_column in table.hidden_columns_in_groups:
                                if (hidden_column == cell.column.name):
                                    cell.value = ''

                        sort_table.rows.append(child_row)
                        end_row_group_index+=1

                    current_row_index += 1

                if (start_row_group_index != -1 and end_row_group_index != -1):
                    if (start_row_group_index != end_row_group_index):
                        sort_table.group_rows.append([start_row_group_index, end_row_group_index])
                current_group_id += 1
            else:
                root_elements = []

                for v in value:
                    root_elements.append(v)

                for root_element in root_elements:
                    row_index, root_row = get_table_row_by_aggreement_number(table, cell_index,
                                                                             root_element.aggreement_value)

                    root_row.is_root = True
                    root_row.group_id = current_group_id
                    sort_table.rows.append(root_row)
                    current_row_index += 1




        calculate_root_elements(sort_table)

        table = copy.deepcopy(sort_table)

        for not_classificated_row in not_classificated_rows:
            not_classificated_row.not_classificated = True
            table.rows.append(not_classificated_row)

        return table
    except Exception as e:
        logging.error("Error. " + str(e))



import logging
import re
import string
import copy
import datetime
# parse xml file
def recursive_navigate(node, paths, index):
    try:
        r = node
        for result in node.findall(paths[index]):
            if (index < len(paths) - 1):
                return recursive_navigate(result, paths, index + 1)
            else:
                return result
        return None
    except Exception as e:
        logging.error("Error. " + str(e))
        return None


# extract all values from single node
def extract_all_items_from_node(node, name):
    try:
        return node.findall(name)

    except Exception as e:
        logging.error("Error. " + str(e))


# check exists node in multiple paths
def check_exists_node(node, paths):
    try:
        result_node = None

        for path in paths:
            result_node = recursive_navigate(node, path, 0)
            if result_node != None:
                break

        return result_node
        pass
    except Exception as e:
        logging.error("Error. " + str(e))
        return None


def convert_str():
    try:

        pass
    except Exception as e:
        pass

# convert output values
def convert_output_values(formats, values):
    try:

        output_values = []
        _value  =''
        for value in values:
            _value = copy.copy(value)

            for f in formats:
                index =0

                #only numbers
                if (f.value == 1):
                    if (_value == ''):

                        _value = '0'
                    # convert to float
                    _value = float(re.sub('[^0-9.]', "", _value.replace(",", ".")))

                #date
                if (f.value==2):
                    pass
                    # if (_value!=''):
                    #     lst = str(_value).split(' ')
                    #
                    #     if (len(lst)>0):
                    #         s_date = lst[0]
                    #         if (s_date!=''):
                    #             date = datetime.datetime.strptime(s_date,'dd.mm.yyyy')


                #thousand div
                if (f.value == 3):
                    if (_value == ''):
                        _value = 0
                    _value = round(_value / 1000, 2)

                index+=1


            output_values.append(_value)



        pass

        t = 0
        return output_values
    except Exception as e:
        logging.error("Error. " + str(e))
        return values


# extracted value join
def extract_value_join(column, values):
    try:
        value = ''
        if (column.extract_value_type.value == 0):
            for v in values:
                value += str(v)

            pass
        elif (column.extract_value_type.value == 1):
            index = 0
            for v in values:
                value += str(v)
                if (index < len(values) - 1):
                    value += ' '
                index += 1

            pass
        elif (column.extract_value_type.value == 2):
            index = 0
            for v in values:
                value += str(v)
                if (index < len(values) - 1):
                    value += '\n'
                index += 1

        # for value in values:

        return value
        pass
    except Exception as e:
        logging.error("Error. " + str(e))
        return None


# extract value from field non table content
def extract_data_from_extra_table_field(root, column):
    try:
        # проверяем пути в в основной секции, если они есть
        column_paths = column.value_paths
        value_attribute_name = column.value_attribute_name
        if (column_paths != None and len(column_paths) > 0):

            paths_to_values = []
            # путь основной, если в массиве только один элемент
            if (len(column_paths) == 1):
                paths_to_values = column_paths[0]
            else:
                # иначе необходимо проверить наличие первой попавшейс секции

                for column_path in column_paths:
                    result_path = check_exists_node(root, column_path)
                    if (result_path != None):
                        paths_to_values = column_path
                        break

            values = []
            # вытягиваем значения и записываем их в словарь
            for path_value in paths_to_values:
                node_value = recursive_navigate(root, path_value, 0)

                if (node_value != None):
                    value = node_value.get(value_attribute_name)

                    if (value != None):
                        values.append(value)

            extracted_value = extract_value_join(column, values)

            # здесь еще возможны алгоритмы преобразования конвертации и т.д.


            return extracted_value

        else:
            # если путей в столбце нет
            pass

        return ""

        pass
    except Exception as e:
        logging.error("Error. " + str(e))
        return ""


# extract value from xml table
def extract_data_from_table_row(xml_row, column):
    try:
        extracted_value = ''
        # проверяем пути в в основной секции, если они есть
        column_paths = column.value_paths
        value_attribute_name = column.value_attribute_name
        if (column_paths != None and len(column_paths) > 0):

            paths_to_values = []
            # путь основной, если в массиве только один элемент
            if (len(column_paths) == 1):
                paths_to_values = column_paths[0]
            else:
                # иначе необходимо проверить наличие первой попавшейс секции

                for column_path in column_paths:
                    result_path = check_exists_node(xml_row, column_path)
                    if (result_path != None):
                        paths_to_values = column_path
                        break

            values = []
            # вытягиваем значения и записываем их в словарь
            for path_value in paths_to_values:

                if (column.condition.is_multiple_values == False):
                    node_value = recursive_navigate(xml_row, path_value, 0)

                    if (node_value != None):
                        value = node_value.get(value_attribute_name)

                        if (value != None):
                            values.append(value)

                else:
                    node_values = extract_all_items_from_node(xml_row, path_value[0])

                    for node_value in node_values:
                        if (node_value != None):
                            value = node_value.get(value_attribute_name)

                            if (value != None):
                                values.append(value)

            if (column.condition.output_value_formats != None and len(column.condition.output_value_formats) > 0):
                values =convert_output_values(column.condition.output_value_formats, values)
                pass

            # convertation if needed
            extracted_value = extract_value_join(column, values)

        return extracted_value
        pass
    except Exception as e:
        logging.error("Error. " + str(e))
        return ""



import json
import uuid
import logging


# content to export sheet cell
class ExportDocumentReportCellContent():
    # constructor
    def __init__(self, data, bgc='#ffffff', color="#000000", ta="left", fz="10", fm="", fw="normal", fs="normal", u="",
                 ff="arial", width=50, id=-1, row=-1, show_grid=False, contains_data=False, height=30):
        try:
            self.width = width
            self.data = data
            self.bgc = bgc
            self.color = color
            self.ta = ta
            self.fz = fz
            self.fm = fm
            self.fw = fw
            self.fs = fs
            self.u = u
            self.ff = ff
            self.id = id
            self.ww = 'break-word'
            self.ws = 'pre-line'
            self.va = 'middle'
            self.ta = 'center'
            self.height = height

            self.bls = 'solid'
            self.brs = 'solid'
            self.bts = 'solid'
            self.bbs = 'solid'

            self.blc = '#7f7f7f'
            self.brc = '#7f7f7f'
            self.btc = '#7f7f7f'
            self.bbc = '#7f7f7f'

            self.blt = 'solid'
            self.brt = 'solid'
            self.btt = 'solid'
            self.bbt = 'solid'
            # self.bgc = bgc
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export document sheet cell
class ExportDocumentReportCell():
    # constructor
    def __init__(self, sheet, row, col, cellJson):
        try:
            self.sheet = sheet
            self.row = row
            self.col = col
            self.json = cellJson
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export document group
class ExportDocumentGroup():
    # constructor
    def __init__(self, level, span):
        try:
            self.level = level
            self.span = span
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # convert model to JSON
    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


# export documnet report floatings model
class ExportDocumentReportFloatings():
    # constructor
    def __init__(self, sheet, name, ftype, json):
        self.sheet = sheet
        self.name = name
        self.ftype = ftype
        self.json = json
        pass


# export sheet model report
class ExportDocumentReportSheet():
    # constructor
    def __init__(self, id, name):
        try:
            self.id = id
            self.name = name
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export sheet report document
class ExportDocumentReportModel():
    # constructor
    def __init__(self, fileName):
        try:
            self.fileName = fileName
            self.sheets = []
            self.floatings = []
            self.cells = []
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # start from 1
    def add_sheet(self, id, name):
        try:
            self.sheets.append(ExportDocumentReportSheet(id, name))
        except Exception as e:
            logging.error("Error " + str(e))

    # convert model to JSON
    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)



from models.settings import config
import click
import modules.shell_starter_modules.shell_starter_net_state_module as ns_module
import modules.shell_starter_modules.shell_starter_general_module as general_module
import modules.shell_starter_modules.shell_starter_data_parse_module as parse_module
cli = click.CommandCollection(
    sources=
    [
        general_module.general_commands_cli,
        ns_module.net_stats_commands_cli,
        parse_module.data_parse_commands_cli
    ]
)



import click
import modules.net_state_modules.net_state_module as ns_module

@click.group()
def net_stats_commands_cli():
    pass


@net_stats_commands_cli.command()
def get_net_state():
    try:
        click.echo(ns_module.get_net_state_info())
        pass
    except Exception as e:
        pass



from models.export_models.export_document_elements import ExportElementStyle
import logging
row_title_style =None
row_value_style = None

table_header_style = None
table_rows_content_style = None
table_rows_root_content_style = None
table_not_classificated_rows = None

#init styles for export
def init_styles():
    try:
        global row_title_style
        row_title_style = ExportElementStyle(fw="bold")

        global row_value_style
        row_value_style = ExportElementStyle()

        global table_header_style
        table_header_style = ExportElementStyle(fw="bold",bgc="#e6e6ff")

        global table_rows_content_style
        table_rows_content_style = ExportElementStyle()

        global table_rows_root_content_style
        table_rows_root_content_style = ExportElementStyle(fw="bold",bgc="#e6f7ff")

        global table_not_classificated_rows
        table_not_classificated_rows = ExportElementStyle(fw="bold",bgc="#ffb3b3", fs="italic")


        logging.info("Styles init successful")
    except Exception as e:
        logging.error("Error initialization. " + str(e))


import click
import models.settings.config as config


@click.group()
def general_commands_cli():
    pass


#get to version code
@general_commands_cli.command()
def version():
    click.echo(config.version_code
               )

#get to build version code
@general_commands_cli.command()
def build_version():
    click.echo(config.build_version_code
               )

#get to version code
@general_commands_cli.command()
def info():
    config.show_system_info()

#get to version name
@general_commands_cli.command()
def version_name():
    try:
        click.echo(config.get_version_name())

        pass
    except Exception as e:
        click.echo(str(e)
                   )


@general_commands_cli.command()
@click.option('--name', required=True, type=str)
@click.option('--age', required=False, type=int)
def input_params(name, age):
    click.echo("Name = " + name + "; age = " + str(age)
               )



from enum import Enum
import logging


# collection of element types
class ExportDocumentElementTypesCollection(Enum):
    ROW = 0
    TABLE = 1
    STATIC_ROW = 2
    EMPTY_ROW = 3


# convert value type
class ExportCellValueConvert(Enum):
    NO_CONVERT = 0
    EXTRACT_DATE = 1
    THOUSAND_FORMAT = 2
    CLEAR_NOT_NUMBER = 3
    MONEY_FORMAT = 4


# export cell style
class ExportElementStyle():
    # constructor
    def __init__(self, bgc="#ffffff", color="#000000", ta="left", fz="8", fm="", fw="normal", fs="normal", u="",
                 ff="arial", width=50, row=-1):
        try:
            if (bgc!='#ffffff'):
                t=0
            self.width = width
            self.bgc = bgc
            self.color = color
            self.ta = ta
            self.fz = fz
            self.fm = fm
            self.fw = fw
            self.fs = fs
            self.u = u
            self.ff = ff
            if (row > 0):
                self.bls = 'solid'
                self.brs = 'solid'
                self.bts = 'solid'
                self.bbs = 'solid'

                self.blc = '#7f7f7f'
                self.brc = '#7f7f7f'
                self.btc = '#7f7f7f'
                self.bbc = '#7f7f7f'

                self.blt = 'solid'
                self.brt = 'solid'
                self.btt = 'solid'
                self.bbt = 'solid'

        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export document cell
class ExportDocumentElementCell():
    # constrcutor
    def __init__(self, navigate_param):
        try:

            self.value_attribute = navigate_param.value_attribute
            self.paths = []
            for path in navigate_param.paths:
                self.paths.append(path)

            self.convert_types = []
            for convert_type in navigate_param.convert_types:
                self.convert_types.append(ExportCellValueConvert(convert_type))

            self.style = navigate_param.style
            self.value = ''

        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # set cell value
    def set_cell_value(self, value):
        try:
            # convert methods
            self.value = value
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # set style to cell
    def set_cell_style(self, bgc="#ffffff", color="#000000", ta="left", fz="8", fm="", fw="normal", fs="normal", u="",
                       ff="arial", width=50, row=-1):
        try:

            self.style = ExportElementStyle(bgc=bgc, color=color, ta=ta, fz=fz, fm=fm, fw=fw, fs=fs, u="", ff=ff,
                                            width=width, row=row)
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export document element row
class ExportDocumentElementRow():
    # constrcutor
    def __init__(self):
        try:
            self.index = -1
            self.cells = []
            self.style = None
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # set row style
    def set_row_style(self, bgc="#ffffff", color="#000000", ta="left", fz="10", fm="", fw="normal", fs="normal", u="",
                      ff="arial", width=50, row=-1):
        try:
            self.style = ExportElementStyle(bgc=bgc, color=color, ta=ta, fz=fz, fm=fm, fw=fw, fs=fs, u="", ff=ff,
                                            width=width, row=row)
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # set row cell styles
    def set_row_cells_style(self):
        try:
            for cell in self.cells:
                cell.set_cell_style( bgc=self.style.bgc, color=self.style.color, ta=self.style.ta, fz=self.style.fz, fm=self.style.fm, fw=self.style.fw, fs=self.style.fs, u=self.style.u,
                       ff=self.style.ff, width=self.style.width, row=-1)
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # set init cells
    def init_value_cells(self, navigate_params, values):
        try:
            for value in values:
                self.cells.append(ExportDocumentElementCell(navigate_params))
                last_cell = self.cells[len(self.cells) - 1]
                last_cell.set_cell_value(value)
        except Exception as e:
            logging.error("Error initialization. " + str(e))


# export document element table
class ExportDocumentElementTable():
    # constructor
    def __init__(self, source_path):
        try:
            self.source_path = source_path
            self.header_row = None
            self.header_source = None

            self.rows = []
            self.rows_source = None
            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))
            pass


# export document element
class ExportDocumentElement():
    # constructor
    def __init__(self, type_id, name, title=""):
        try:
            self.type = ExportDocumentElementTypesCollection(type_id)
            self.name = name
            self.title = title
            self.table = None
            self.row = None
            self.include_in_report = True
        except Exception as e:
            logging.error("Error initialization. " + str(e))
            pass

    # init element table
    def init_element_table(self, navigate_params):
        try:
            pass
        except Exception as e:
            logging.error("Error initialization. " + str(e))

    # init element row
    def init_element_row(self, navigate_params):
        try:
            self.row = ExportDocumentElementRow()
            for navigate_param in navigate_params:
                self.row.cells.append(
                    ExportDocumentElementCell(navigate_param)
                )
        except Exception as e:
            logging.error("Error initialization. " + str(e))
